@@ -5,9 +5,10 @@ use strict;
 use warnings;
 use Data::Dumper;
 use DBI;
-use Helios::Worker;
+use Helios::Service;
+use Helios::LogEntry::Levels ':all';
 
-our $VERSION = '1.29_01';
+our $VERSION = '1.44_5032';
 our $dbh;
 our %config;
 
@@ -17,10 +18,21 @@ if (defined($ENV{HELIOS_INI}) ) {
 } else {
     $inifile = './helios.ini';
 }
-my $worker = new Helios::Worker;
-%config = $worker->getParamsFromIni($inifile);
+my $worker = new Helios::Service;
+%config = $worker->getConfigFromIni($inifile);
 
-$dbh = DBI->connect($config{dsn}, $config{user}, $config{password});
+# This section of the software is Copyright (C) 2011 by Andrew Johnson.
+# See copyright notice at the end of this file for license information.
+	# we need to support db options here too
+	my $optext = $config{options};
+	my $dbopt = eval "{ $optext }";
+	if ($@) {
+		# we're just going to log a warning and ignore the option
+		$worker->logMsg(LOG_WARNING, __PACKAGE__.': Invalid options specified in config: '.$optext);
+		$dbopt = undef;
+	}
+# End code under Andrew Johnson copyright.
+$dbh = DBI->connect($config{dsn}, $config{user}, $config{password}, $dbopt);
 
 =head1 NAME
 
@@ -339,16 +351,18 @@ __END__
 
 =head1 SEE ALSO
 
-L<Helios::Panoptes>, L<Helios::Worker>, L<helios.pl>, <CGI::Application>, L<HTML::Template>
+L<Helios::Panoptes>, L<Helios::Service>, L<helios.pl>, <CGI::Application>, L<HTML::Template>
 
 =head1 AUTHOR 
 
-Andrew Johnson, <ajohnson at ittoolbox dotcom>
-Ben Kucenski, <bkucenski at ittoolbox dotcom>
+Andrew Johnson, <lajandy at cpan dotorg>
+Ben Kucenski, <bkucenski at toolbox dotcom>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2008 by CEB Toolbox, Inc.
+Copyright (C) 2008-9 by CEB Toolbox, Inc.
+
+Portions of this software, where noted, are Copyright (C) 2011 by Andrew Johnson.
 
 This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself, either Perl version 5.8.0 or, at your option, any later version of Perl 5 you may have available.
 
@@ -356,4 +370,4 @@ This library is free software; you can redistribute it and/or modify it under th
 
 This software comes with no warranty of any kind.
 
-=cut
\ No newline at end of file
+=cut
